@@ -28,7 +28,7 @@
 "*helpURL: http://www.jwz.org/xscreensaver/man.html",
 "*loadURL: netscape -remote 'openURL(%s)' || netscape '%s'",
 "*manualCommand: xterm +sb -fg black -bg gray75 -T '%s manual' \
-        -e /bin/sh -c 'man \"%s\" || read'",
+        -e /bin/sh -c 'man \"%s\" || read foo'",
 "*dateFormat:		%d-%b-%y (%a); %I:%M %p",
 "*installColormap:	True",
 "*programs:								      \
@@ -222,32 +222,90 @@
 "*splash.help.label:		Help",
 "*fontList:                       *-helvetica-medium-r-*-*-*-120-*-*-*-iso8859-1",
 "*demoDialog*label1.fontList:     *-helvetica-medium-r-*-*-*-140-*-*-*-iso8859-1",
-"*XmTextField.fontList:             *-courier-medium-r-*-*-*-120-*-*-*-iso8859-1",
+"*cmdText.fontList:                 *-courier-medium-r-*-*-*-120-*-*-*-iso8859-1",
 "*label0.fontList:                  *-helvetica-bold-r-*-*-*-140-*-*-*-iso8859-1",
-"XScreenSaver*XmList.fontList:      *-courier-medium-r-*-*-*-120-*-*-*-iso8859-1",
-"*XmDialogShell*foreground:		#000000",
-"*XmDialogShell*background:		#E5E5E5",
-"*XmDialogShell*XmTextField.foreground:	#000000",
-"*XmDialogShell*XmTextField.background:	#FFFFFF",
-"*XmDialogShell*demoList.foreground:	#000000",
-"*XmDialogShell*demoList.background:	#FFFFFF",
-"*XmDialogShell.title:		XScreenSaver",
-"*versionWarning_popup.title:	XScreenSaver Warning",
-"*demoForm_popup.title:		XScreenSaver Demo",
-"*preferencesForm_popup.title:	XScreenSaver Preferences",
+"*doc.fontList:                   *-helvetica-medium-r-*-*-*-100-*-*-*-iso8859-1",
+"*foreground:			#000000",
+"*background:			#C0C0C0",
+"*XmTextField.foreground:	#000000",
+"*XmTextField.background:	#FFFFFF",
+"*list.foreground:		#000000",
+"*list.background:		#FFFFFF",
+"*ApplicationShell.title:	XScreenSaver",
+"*warning.title:			XScreenSaver",
+"*warning_popup.title:		XScreenSaver",
 "*allowShellResize:		True",
 "*autoUnmanage:			False",
-"*demoDialog.maxWidth:		600",
-"*label1.labelString:		XScreenSaver %s",
-"*label1.label:			XScreenSaver %s",
-"*label2.labelString: Copyright  1991-1999 by Jamie Zawinski <jwz@jwz.org>",
-"*label2.label:	     Copyright  1991-1999 by Jamie Zawinski <jwz@jwz.org>",
-"*demoList.visibleItemCount:	10",
-"*demoList.automaticSelection:	True",
-"*next.labelString:		Run Next",
-"*prev.labelString:		Run Previous",
-"*edit.labelString:		Preferences",
-"*restart.labelString:		Reinitialize",
+"*menubar*file.labelString:	File",
+"*menubar*file.mnemonic:		F",
+"*file.blank.labelString:	Blank Screen Now",
+"*file.blank.mnemonic:		B",
+"*file.lock.labelString:		Lock Screen Now",
+"*file.lock.mnemonic:		L",
+"*file.kill.labelString:		Kill Daemon",
+"*file.kill.mnemonic:		K",
+"*file.restart.labelString:	Restart Daemon",
+"*file.restart.mnemonic:		R",
+"*file.exit.labelString:		Exit",
+"*file.exit.mnemonic:		E",
+"*menubar*edit.labelString:	Edit",
+"*menubar*edit.mnemonic:		E",
+"*edit.cut.labelString:		Cut",
+"*edit.cut.mnemonic:		u",
+"*edit.copy.labelString:		Copy",
+"*edit.copy.mnemonic:		C",
+"*edit.paste.labelString:	Paste",
+"*edit.paste.mnemonic:		P",
+"*menubar*help.labelString:	Help",
+"*menubar*help.mnemonic:		H",
+"*help.about.labelString:	About...",
+"*help.about.mnemonic:		A",
+"*help.docMenu.labelString:	Documentation...",
+"*help.docMenu.mnemonic:		D",
+"*demoTab.marginWidth:		10",
+"*optionsTab.marginWidth:	10",
+"*XmScrolledWindow.topOffset:	10",
+"*XmScrolledWindow.leftOffset:	10",
+"*demoTab.topOffset:		4",
+"*form1.bottomOffset:		10",
+"*form3.leftOffset:		10",
+"*form3.rightOffset:		10",
+"*frame.topOffset:		10",
+"*frame.bottomOffset:		10",
+"*enabled.topOffset:		10",
+"*visLabel.topOffset:		10",
+"*combo.topOffset:		10",
+"*form4.bottomOffset:		4",
+"*hr.bottomOffset:		4",
+"*XmComboBox.marginWidth:	0",
+"*XmComboBox.marginHeight:	0",
+"*demo.marginWidth:		30",
+"*demo.marginHeight:		4",
+"*man.marginWidth:		10",
+"*man.marginHeight:		4",
+"*down.leftOffset:		40",
+"*down.marginWidth:		4",
+"*down.marginHeight:		4",
+"*up.marginWidth:		4",
+"*up.marginHeight:		4",
+"*frame.traversalOn:		False",
+"*list.automaticSelection:	True",
+"*list.visibleItemCount:		20",
+"*doc.columns:			60",
+"*combo.columns:			11",
+"*demoTab.labelString:		Graphics Demos",
+"*optionsTab.labelString:	Screensaver Options",
+"*down.labelString:		\\\\/ ",
+"*up.labelString:		/\\\\ ",
+"*frameLabel.labelString:	",
+"*cmdLabel.labelString:		Command Line:",
+"*cmdLabel.alignment:		ALIGNMENT_BEGINNING",
+"*enabled.labelString:		Enabled",
+"*visLabel.labelString:		Visual:",
+"*visLabel.alignment:		ALIGNMENT_END",
+"*visLabel.leftOffset:		20",
+"*demo.labelString:		Demo",
+"*man.labelString:		Documentation...",
 "*done.labelString:		Quit",
 "*preferencesLabel.labelString:	XScreenSaver Parameters",
 "*timeoutLabel.labelString:	Saver Timeout",
@@ -262,84 +320,14 @@
 "*fadeToggle.labelString:	Fade Colormap",
 "*unfadeToggle.labelString:	Unfade Colormap",
 "*lockToggle.labelString:	Require Password",
-"*preferencesDone.labelString:	OK",
-"*preferencesCancel.labelString:	Cancel",
-"XScreenSaver*dragInitiatorProtocolStyle: DRAG_NONE",
-"XScreenSaver*dragReceiverProtocolStyle:  DRAG_NONE",
-"*demo_dialog.title:		XScreenSaver Demo",
-"*preferences_dialog.title:	XScreenSaver Preferences",
-"*warning_dialog.title:		XScreenSaver Warning",
-"*demo_dialog.geometry:		=640x400",
-"*demo_dialog*font:		*-helvetica-bold-r-*-*-*-120-*-*-*-iso8859-1",
-"*preferences_dialog*font:	*-helvetica-bold-r-*-*-*-120-*-*-*-iso8859-1",
-"*demo_dialog*label1.font:	*-helvetica-bold-r-*-*-*-140-*-*-*-iso8859-1",
-"*preferences_dialog*label1.font:*-helvetica-bold-r-*-*-*-140-*-*-*-iso8859-1",
-"XScreenSaver*warning_dialog*label0.font:	\
-				*-helvetica-bold-r-*-*-*-140-*-*-*-iso8859-1",
-"XScreenSaver*warning_dialog*Label.font:	\
-				*-helvetica-bold-r-*-*-*-120-*-*-*-iso8859-1",
-"XScreenSaver*warning_dialog*Command.font: \
-				*-helvetica-bold-r-*-*-*-140-*-*-*-iso8859-1",
-"XScreenSaver.demo_dialog*List.font:	\
-				*-courier-medium-r-*-*-*-120-*-*-*-iso8859-1",
-"XScreenSaver.demo_dialog*Text*font:	\
-				*-courier-medium-r-*-*-*-120-*-*-*-iso8859-1",
-"XScreenSaver.demo_dialog*foreground:			#000000",
-"XScreenSaver.demo_dialog*background:			#E5E5E5",
-"XScreenSaver.demo_dialog*List.background:		#FFFFFF",
-"XScreenSaver.demo_dialog*Scrollbar.background:		#D9D9D9",
-"XScreenSaver.demo_dialog*Command.background:		#D9D9D9",
-"XScreenSaver.demo_dialog*Text*background:		#FFFFFF",
-"XScreenSaver.preferences_dialog*foreground:		#000000",
-"XScreenSaver.preferences_dialog*background:		#E5E5E5",
-"XScreenSaver.preferences_dialog*Command.background:	#D9D9D9",
-"XScreenSaver.preferences_dialog*Toggle.background:	#D9D9D9",
-"XScreenSaver.preferences_dialog*Text*background:	#FFFFFF",
-"XScreenSaver.warning_dialog*foreground:			#000000",
-"XScreenSaver.warning_dialog*background:			#E5E5E5",
-"XScreenSaver.warning_dialog*Command.background:		#D9D9D9",
-"*preferences_dialog*Dialog.value.translations: #override\\n\
-	<Key>Return: beginning-of-line()\\n",
-"*demo_dialog*viewport.height:			200",
-"*Form.borderWidth:				0",
-"*Box.borderWidth:				0",
-"*Label.borderWidth:				0",
-"*preferences_dialog*Dialog.borderWidth:		0",
-"*demo_dialog*run.label:				Run",
-"*demo_dialog*next.label:			Run Next",
-"*demo_dialog*prev.label:			Run Previous",
-"*demo_dialog*edit.label:			Preferences",
-"*demo_dialog*restart.label:			Reinitialize",
-"*demo_dialog*done.label:			Quit",
-"XScreenSaver.demo_dialog*Command.internalWidth:  10",
-"XScreenSaver.demo_dialog*Command.internalHeight: 4",
-"*preferences_dialog*timeout.label:		Saver Timeout:",
-"*preferences_dialog*cycle.label:		Cycle Timeout:",
-"*preferences_dialog*fade.label:			Fade Duration:",
-"*preferences_dialog*ticks.label:		Fade Ticks:",
-"*preferences_dialog*lockTime.label:		Lock Timeout:",
-"*preferences_dialog*passwdTime.label:		Password Timeout:",
-"XScreenSaver.preferences_dialog*Command.internalWidth:  10",
-"XScreenSaver.preferences_dialog*Command.internalHeight: 4",
-"*preferences_dialog*label1.label:		XScreenSaver Parameters",
-"*preferences_dialog*buttonbox.verbose.label:	Verbose",
-"*preferences_dialog*buttonbox.cmap.label:	Install Colormap",
-"*preferences_dialog*buttonbox.fade.label:	Fade Colormap",
-"*preferences_dialog*buttonbox.unfade.label:	Unfade Colormap",
-"*preferences_dialog*buttonbox.lock.label:	Require Password",
-"*preferences_dialog*done.label:			Ok",
-"*preferences_dialog*cancel.label:		Cancel",
-"*warning_dialog*ok.label:			Ok",
-"*warning_dialog*horizDistance:			30",
-"*warning_dialog*vertDistance:			0",
-"*warning_dialog*Label.internalWidth:		1",
-"*warning_dialog*Label.internalHeight:		0",
-"*warning_dialog*label0.horizDistance:		80",
-"*warning_dialog*label0.vertDistance:		20",
-"*warning_dialog*Command.horizDistance:		160",
-"*warning_dialog*Command.vertDistance:		20",
-"*warning_dialog*Command.internalWidth:		20",
-"*warning_dialog*Command.internalHeight:		5",
+"*OK.marginWidth:		30",
+"*OK.marginHeight:		4",
+"*OK.leftOffset:			10",
+"*OK.bottomOffset:		10",
+"*Cancel.marginWidth:		30",
+"*Cancel.marginHeight:		4",
+"*Cancel.rightOffset:		10",
+"*Cancel.bottomOffset:		10",
 "*hacks.documentation.isInstalled: True",
 "*hacks.grav.documentation:						\
 This program draws a simple orbital simulation.	 If you turn on		\
